@@ -1,9 +1,11 @@
 import copy
 import dis
+import sys
 import types
 import typing
 import simplejson
 
+import bytecodemanipulation.assembler.util.tokenizer
 from bytecodemanipulation.Opcodes import HAS_CELL_VARIABLE
 from bytecodemanipulation.Opcodes import HAS_JUMP_BACKWARDS
 from bytecodemanipulation.Opcodes import (
@@ -43,12 +45,26 @@ class Instruction:
         "arg",
         "_next_instruction",
         "previous_instructions",
+        "source_location",
     )
 
     @classmethod
     def create(cls, *args, **kwargs):
         return cls(None, -1, *args, **kwargs)
 
+    @classmethod
+    def create_with_token(
+        cls,
+        token: bytecodemanipulation.assembler.util.tokenizer.AbstractToken,
+        function: typing.Optional["MutableFunction"],
+        offset: int | None,
+        opcode_or_name: int | str,
+        arg_value: object = None,
+        arg: int = None,
+        _decode_next=True,
+    ) -> "Instruction":
+        raise NotImplementedError("not bound!")
+
     def __init__(
         self,
         function: typing.Optional["MutableFunction"],
@@ -57,12 +73,14 @@ class Instruction:
         arg_value: object = None,
         arg: int = None,
         _decode_next=True,
+        pos_info=None,
     ):
         self.function = function
         self.offset = offset
         self.opcode, self.opname = self._pair_instruction(opcode_or_name)
         self.arg_value = arg_value
         self.arg = arg
+        self.source_location = pos_info
 
         if (
             self.arg is not None
@@ -70,7 +88,9 @@ class Instruction:
             and (_decode_next or not self.has_jump())
         ):
             self.change_arg(self.arg)
-        elif self.arg_value is not None and self.arg is None:
+        elif (
+            self.arg_value is not None or self.opcode == Opcodes.LOAD_CONST
+        ) and self.arg is None:
             self.change_arg_value(self.arg_value)
 
         # Reference to the next instruction
@@ -86,6 +106,21 @@ class Instruction:
 
         self.previous_instructions: typing.List["Instruction"] | None = None
 
+    def copy(self, owner: "MutableFunction" = None) -> "Instruction":
+        instance = type(self)(
+            self.function,
+            self.offset,
+            self.opcode,
+            self.arg_value
+            if self.arg_value is not None or self.opcode == Opcodes.LOAD_CONST
+            else self.arg,
+        )
+
+        if owner:
+            instance.update_owner(owner, -1, force_change_arg_index=True)
+
+        return instance
+
     def apply_visitor(
         self,
         visitor: AbstractInstructionWalker,
@@ -94,16 +129,54 @@ class Instruction:
         if visited is None:
             visited = set()
 
+        if self in visited:
+            return
+
+        visited.add(self)
+
         visitor.visit(self)
 
         if self.has_stop_flow():
             return
 
-        self.next_instruction.apply_visitor(visitor, visited)
+        if self.next_instruction is not None and self.next_instruction != self:
+            self.next_instruction.apply_visitor(visitor, visited)
 
-        if self.has_jump():
+        if self.has_jump() and isinstance(self.arg_value, Instruction):
             typing.cast(Instruction, self.arg_value).apply_visitor(visitor, visited)
 
+    def apply_value_visitor(
+        self,
+        callback: typing.Callable[
+            ["Instruction", typing.Any | None, typing.Any | None], typing.Any
+        ],
+        visited: typing.Dict["Instruction", typing.Any] = None,
+    ) -> typing.Any:
+        if visited is None:
+            visited = {}
+
+        elif self in visited:
+            return visited[self]
+
+        visited[self] = None
+
+        if self.has_stop_flow():
+            return callback(self, None, None)
+
+        if self.has_jump() and isinstance(self.arg_value, Instruction):
+            return callback(
+                self,
+                self.next_instruction.apply_value_visitor(callback, visited),
+                self.arg_value.apply_value_visitor(callback, visited),
+            )
+
+        if self.next_instruction is not None:
+            return callback(
+                self, self.next_instruction.apply_value_visitor(callback, visited), None
+            )
+
+        return callback(self, None, None)
+
     def add_previous_instruction(self, instruction: "Instruction"):
         if self.previous_instructions is None:
             self.previous_instructions = [instruction]
@@ -169,25 +242,33 @@ class Instruction:
 
         return (
             self.opcode == other.opcode
-            and (self.offset == other.offset or self.offset is None or other.offset is None)
             and (
-                (self.arg_value == other.arg_value) if not isinstance(self.arg_value, Instruction) else (id(self.arg_value) == id(other.arg_value))
+                self.offset == other.offset
+                or self.offset is None
+                or other.offset is None
+            )
+            and (
+                (self.arg_value == other.arg_value)
+                if not isinstance(self.arg_value, Instruction)
+                else (id(self.arg_value) == id(other.arg_value))
                 if self.arg_value is not None and other.arg_value is not None
                 else True
             )
         )
 
-    def lossy_eq(self, other):
+    def lossy_eq(self, other: "Instruction") -> bool:
         if not isinstance(other, Instruction):
             return False
 
-        return (
-            self.opcode == other.opcode
-            and (
-                (self.arg_value == other.arg_value) if not isinstance(self.arg_value, Instruction) else self.arg_value.lossy_eq(other.arg_value)
-                if self.arg_value is not None and other.arg_value is not None
-                else True
+        return self.opcode == other.opcode and (
+            (
+                (self.arg_value == other.arg_value)
+                if not isinstance(self.arg_value, Instruction)
+                else self.arg_value.lossy_eq(other.arg_value)
             )
+            if (self.arg_value is not None and other.arg_value is not None)
+            or self.opcode == Opcodes.LOAD_CONST
+            else True
         )
 
     def __hash__(self):
@@ -196,23 +277,29 @@ class Instruction:
     def get_arg(self):
         return 0 if self.arg is None else self.arg
 
-    def change_opcode(self, opcode: int | str):
+    def change_opcode(self, opcode: int | str, arg_value=None, update_next=True):
         self.opcode, self.opname = self._pair_instruction(opcode)
         # todo: what happens with the arg?
 
-        self.next_instruction = (
-            None
-            if self.function is None
-            or self.offset is None
-            or self.opcode in END_CONTROL_FLOW
-            or self.offset == -1
-            else self.function.instructions[self.offset + 1]
-        )
+        if update_next:
+            self.next_instruction = (
+                None
+                if self.function is None
+                or self.offset is None
+                or self.opcode in END_CONTROL_FLOW
+                or self.offset == -1
+                else self.function.instructions[self.offset + 1]
+            )
 
         if self.opcode < dis.HAVE_ARGUMENT:
             self.arg = 0
             self.arg_value = None
 
+        if arg_value:
+            self.change_arg_value(arg_value)
+
+        return self
+
     def change_arg_value(self, value: object):
         self.arg_value = value
 
@@ -220,17 +307,20 @@ class Instruction:
             if self.opcode in HAS_NAME:
                 assert isinstance(value, str)
                 self.arg = self.function.allocate_shared_name(value)
+            elif self.opcode in HAS_CELL_VARIABLE:
+                assert isinstance(value, str)
+                self.arg = self.function.allocate_shared_cell(value)
             elif self.opcode in HAS_CONST:
                 self.arg = self.function.allocate_shared_constant(value)
             elif self.opcode in HAS_LOCAL:
                 assert isinstance(value, str), (value, self.opname)
                 self.arg = self.function.allocate_shared_variable_name(value)
             elif self.opcode in HAS_JUMP_ABSOLUTE:
-                assert isinstance(value, Instruction), value
-                self.arg = value.offset
+                if isinstance(value, Instruction):
+                    self.arg = value.offset
             elif self.opcode == Opcodes.FOR_ITER:
-                assert isinstance(value, Instruction), value
-                self.arg = value.offset - self.offset
+                if isinstance(value, Instruction):
+                    self.arg = value.offset - self.offset
             elif self.opcode in HAS_JUMP_FORWARD:
                 assert isinstance(value, Instruction), value
                 self.arg = value.offset - self.offset
@@ -242,8 +332,18 @@ class Instruction:
 
         if self.function is not None:
             try:
-                if self.opcode in HAS_NAME:
+                flag = False
+                if sys.version_info.minor >= 11:
+                    if self.opcode == Opcodes.LOAD_GLOBAL:
+                        self.arg_value = self.function.shared_names[arg >> 1]
+                        flag = True
+
+                if flag:
+                    pass
+                elif self.opcode in HAS_NAME:
                     self.arg_value = self.function.shared_names[arg]
+                elif self.opcode in HAS_CELL_VARIABLE:
+                    self.arg_value = self.function.cell_variables[arg]
                 elif self.opcode in HAS_CONST:
                     self.arg_value = self.function.constants[arg]
                 elif self.opcode in HAS_LOCAL:
@@ -255,7 +355,13 @@ class Instruction:
                 elif self.opcode in HAS_JUMP_FORWARD and self.offset is not None:
                     self.arg_value = self.function.instructions[arg + self.offset]
             except:
-                print(self.opname, arg, self.function.shared_names, self.function.constants, self.function.shared_variable_names)
+                print(
+                    self.opname,
+                    arg,
+                    self.function.shared_names,
+                    self.function.constants,
+                    self.function.shared_variable_names,
+                )
                 raise
         else:
             self.arg_value = None
@@ -295,7 +401,11 @@ class Instruction:
         return self.opcode in END_CONTROL_FLOW
 
     def update_owner(
-        self, function: "MutableFunction", offset: int, update_following=True
+        self,
+        function: "MutableFunction",
+        offset: int,
+        update_following=True,
+        force_change_arg_index=False,
     ):
         previous_function = self.function
 
@@ -305,9 +415,15 @@ class Instruction:
         # If previously the ownership was unset, and we have not fully referenced args, do it now!
         # todo: when previous owner was set, and arg is not None, we might need to de-ref the value
         #    and re-ref afterwards, so the value lives in the new owner
-        if self.arg is not None and self.arg_value is None:
+        if (
+            self.arg is not None
+            and self.arg_value is None
+            and (not force_change_arg_index or self.opcode != Opcodes.LOAD_CONST)
+        ):
             self.change_arg(self.arg)
-        elif self.arg_value is not None and self.arg is None:
+        elif (self.arg_value is not None or self.opcode == Opcodes.LOAD_CONST) and (
+            self.arg is None or force_change_arg_index
+        ):
             self.change_arg_value(self.arg_value)
 
         if update_following:
@@ -338,12 +454,14 @@ class Instruction:
         if visited is None:
             visited = set()
 
-        if self.opcode == Opcodes.NOP and self.next_instruction is not None:
-            return self.next_instruction.optimise_tree(visited)
-
         if self in visited:
             return self
 
+        visited.add(self)
+
+        if self.opcode == Opcodes.NOP and self.next_instruction is not None:
+            return self.next_instruction.optimise_tree(visited)
+
         while self.next_instruction is not None:
             assert isinstance(self.next_instruction, Instruction)
 
@@ -357,8 +475,6 @@ class Instruction:
 
             break
 
-        visited.add(self)
-
         if (
             self.next_instruction is not None
             and not self.has_stop_flow()
@@ -376,7 +492,9 @@ class Instruction:
 
         return self
 
-    def trace_variable_set(self, name: str, visited: typing.Set[str] = None) -> typing.Iterator["Instruction"]:
+    def trace_variable_set(
+        self, name: str, visited: typing.Set[str] = None
+    ) -> typing.Iterator["Instruction"]:
         if visited and self in visited:
             return
 
@@ -403,10 +521,19 @@ class Instruction:
             yield from instr._trace_stack_position(stack_position, set(), self)
 
     def get_priorities_previous(self) -> typing.List["Instruction"]:
-        if not self.previous_instructions: return []
+        if not self.previous_instructions:
+            return []
 
-        real_previous = [instr for instr in self.previous_instructions if instr.next_instruction == self]
-        unreal_previous = [instr for instr in self.previous_instructions if instr.next_instruction != self]
+        real_previous = [
+            instr
+            for instr in self.previous_instructions
+            if instr.next_instruction == self
+        ]
+        unreal_previous = [
+            instr
+            for instr in self.previous_instructions
+            if instr.next_instruction != self
+        ]
         highest = None
 
         for e in real_previous:
@@ -418,7 +545,9 @@ class Instruction:
 
         return [highest] + [e for e in real_previous if e != highest] + unreal_previous
 
-    def trace_normalized_stack_position(self, stack_position: int) -> typing.Optional["Instruction"]:
+    def trace_normalized_stack_position(
+        self, stack_position: int
+    ) -> typing.Optional["Instruction"]:
         target = next(self.trace_stack_position(stack_position))
 
         if target.opcode == Opcodes.DUP_TOP:
@@ -426,7 +555,9 @@ class Instruction:
 
         if target.opcode == Opcodes.LOAD_FAST:
             try:
-                variable_set = next(target.trace_variable_set(typing.cast(str, target.arg_value)))
+                variable_set = next(
+                    target.trace_variable_set(typing.cast(str, target.arg_value))
+                )
             except StopIteration:
                 return target
 
@@ -435,7 +566,10 @@ class Instruction:
         return target
 
     def _trace_stack_position(
-        self, stack_position: int, yielded: typing.Set["Instruction"], previous_instr: "Instruction" = None
+        self,
+        stack_position: int,
+        yielded: typing.Set["Instruction"],
+        previous_instr: "Instruction" = None,
     ) -> typing.Iterator["Instruction"]:
         assert stack_position >= 0
 
@@ -467,7 +601,9 @@ class Instruction:
 
             if additional_pos:
                 for instr in self.get_priorities_previous():
-                    yield from instr._trace_stack_position(additional_pos, yielded, self)
+                    yield from instr._trace_stack_position(
+                        additional_pos, yielded, self
+                    )
 
             return
 
@@ -490,13 +626,17 @@ class Instruction:
     def trace_stack_position_use(
         self, stack_position: int
     ) -> typing.Iterator["Instruction"]:
-        print(self, 0)
+        # print(self, 0)
 
         if not self.has_stop_flow():
-            yield from self.next_instruction._trace_stack_position_use(stack_position, set())
+            yield from self.next_instruction._trace_stack_position_use(
+                stack_position, set()
+            )
 
         if self.has_jump():
-            yield from typing.cast(Instruction, self.arg_value)._trace_stack_position_use(stack_position, set())
+            yield from typing.cast(
+                Instruction, self.arg_value
+            )._trace_stack_position_use(stack_position, set())
 
     def _trace_stack_position_use(
         self, stack_position: int, yielded: typing.Set["Instruction"]
@@ -515,7 +655,9 @@ class Instruction:
             yield self
 
             if additional_pos:
-                yield from self.next_instruction._trace_stack_position_use(additional_pos, yielded)
+                yield from self.next_instruction._trace_stack_position_use(
+                    additional_pos, yielded
+                )
 
             return
 
@@ -523,31 +665,54 @@ class Instruction:
         stack_position += pushed
 
         if not self.has_stop_flow():
-            yield from self.next_instruction._trace_stack_position_use(stack_position, yielded)
+            yield from self.next_instruction._trace_stack_position_use(
+                stack_position, yielded
+            )
 
         if self.has_jump():
-            yield from typing.cast(Instruction, self.arg_value)._trace_stack_position_use(stack_position, yielded)
+            yield from typing.cast(
+                Instruction, self.arg_value
+            )._trace_stack_position_use(stack_position, yielded)
 
         if additional_pos:
-            yield from self.next_instruction._trace_stack_position_use(additional_pos, yielded)
+            yield from self.next_instruction._trace_stack_position_use(
+                additional_pos, yielded
+            )
 
     def get_stack_affect(self) -> typing.Tuple[int, int, int | None]:
         # pushed, popped, additional
-        if self.opcode in (Opcodes.NOP, Opcodes.POP_BLOCK, Opcodes.POP_EXCEPT, Opcodes.SETUP_FINALLY, Opcodes.GEN_START, Opcodes.JUMP_ABSOLUTE):
+        if self.opcode in (
+            Opcodes.NOP,
+            Opcodes.POP_BLOCK,
+            Opcodes.POP_EXCEPT,
+            Opcodes.SETUP_FINALLY,
+            Opcodes.GEN_START,
+            Opcodes.JUMP_ABSOLUTE,
+            Opcodes.BYTECODE_LABEL,
+            # Opcodes.CACHE,
+            # Opcodes.PRECALL,
+            # Opcodes.RESUME,
+        ):
             return 0, 0, None
 
         if self.opcode == Opcodes.DUP_TOP:
             return 2, 1, None
 
-        if self.opcode in (Opcodes.GET_ITER, Opcodes.FOR_ITER):
+        if self.opcode == Opcodes.GET_ITER:
             return 1, 0, None
 
+        if self.opcode == Opcodes.FOR_ITER:
+            return 1, 1, None
+
         if self.opcode in (
             Opcodes.LOAD_CONST,
             Opcodes.LOAD_GLOBAL,
             Opcodes.LOAD_FAST,
             Opcodes.LOAD_DEREF,
             Opcodes.LOAD_CLOSURE,
+            Opcodes.LOAD_BUILD_CLASS,
+            Opcodes.LOAD_ASSERTION_ERROR,
+            Opcodes.LOAD_NAME,
         ):
             return 1, 0, None
 
@@ -560,26 +725,21 @@ class Instruction:
         ):
             return 1, self.arg, None
 
-        if self.opcode in (
-            Opcodes.ROT_TWO,
-        ):
+        if self.opcode in (Opcodes.ROT_TWO,):
             return 2, 2, None
 
-        if self.opcode in (
-            Opcodes.FORMAT_VALUE,
-        ):
+        if self.opcode in (Opcodes.FORMAT_VALUE,):
             return 1, (2 if (self.arg & 0x04) == 0x04 else 1), None
 
         if self.opcode in (
             Opcodes.CALL_FUNCTION,
             Opcodes.CALL_METHOD,
             Opcodes.CALL_FUNCTION_KW,
+            # Opcodes.CALL,
         ):
             return 1, self.arg + 1, None
 
-        if self.opcode in (
-            Opcodes.CALL_FUNCTION_EX,
-        ):
+        if self.opcode in (Opcodes.CALL_FUNCTION_EX,):
             count = 2
 
             if self.arg & 0x01:
@@ -603,6 +763,7 @@ class Instruction:
             Opcodes.DICT_MERGE,
             Opcodes.BINARY_FLOOR_DIVIDE,
             Opcodes.BINARY_ADD,
+            Opcodes.INPLACE_ADD,
             Opcodes.BINARY_SUBTRACT,
             Opcodes.BINARY_MULTIPLY,
             Opcodes.BINARY_TRUE_DIVIDE,
@@ -614,6 +775,7 @@ class Instruction:
             Opcodes.BINARY_POWER,
             Opcodes.INPLACE_SUBTRACT,
             Opcodes.IMPORT_NAME,
+            Opcodes.YIELD_FROM,
         ):
             return 1, 2, None
 
@@ -624,12 +786,17 @@ class Instruction:
             Opcodes.LIST_TO_TUPLE,
             Opcodes.IMPORT_FROM,
             Opcodes.UNARY_NEGATIVE,
+            Opcodes.YIELD_VALUE,
+            Opcodes.GET_YIELD_FROM_ITER,
+            Opcodes.UNARY_NOT,
+            Opcodes.UNARY_NEGATIVE,
+            Opcodes.UNARY_INVERT,
+            Opcodes.UNARY_POSITIVE,
+            Opcodes.STATIC_ATTRIBUTE_ACCESS,
         ):
             return 1, 1, None
 
-        if self.opcode in (
-            Opcodes.STORE_ATTR,
-        ):
+        if self.opcode in (Opcodes.STORE_ATTR,):
             return 2, 0, None
 
         if self.opcode in (
@@ -637,9 +804,9 @@ class Instruction:
             Opcodes.POP_JUMP_IF_TRUE,
             Opcodes.POP_JUMP_IF_FALSE,
             Opcodes.STORE_FAST,
+            Opcodes.STORE_NAME,
             Opcodes.STORE_DEREF,
-            Opcodes.YIELD_VALUE,
-            Opcodes.YIELD_FROM,
+            Opcodes.STORE_GLOBAL,
             Opcodes.RETURN_VALUE,
         ):
             return 0, 1, None
@@ -653,10 +820,23 @@ class Instruction:
         if self.opcode == Opcodes.MAKE_FUNCTION:
             return 1, 2 + self.arg.bit_count(), None
 
+        if self.opcode == Opcodes.RAISE_VARARGS:
+            return 0, self.arg, None
+
+        if self.opcode == Opcodes.DUP_TOP_TWO:
+            return 2, 4, None
+
         raise RuntimeError(self)
 
+    def special_stack_affect_when_followed_by(self, instr: "Instruction") -> int:
+        if self.opcode == Opcodes.FOR_ITER and instr == self.arg_value:
+            return -2
+
+        return 0
+
     def insert_after(self, *instructions: "Instruction" | typing.List["Instruction"]):
-        if not instructions: return self
+        if not instructions:
+            return self
 
         if isinstance(instructions[0], (list, tuple)):
             if len(instructions) > 1:
@@ -664,6 +844,9 @@ class Instruction:
 
             instructions = instructions[0]
 
+        if len(instructions) == 0:
+            return self
+
         instructions[0].next_instruction = self.next_instruction
         self.next_instruction = instructions[0]
 
@@ -687,25 +870,60 @@ class MutableFunction:
 
         self.code_object: types.CodeType = target.__code__
 
-        self.argument_count = self.code_object.co_argcount
-        self.cell_variables = list(self.code_object.co_cellvars)
-        self.__raw_code = bytearray(self.code_object.co_code)
-        self.constants = list(self.code_object.co_consts)
-        self.filename = self.code_object.co_filename
-        self.first_line_number = self.code_object.co_firstlineno
-        self.code_flags = self.code_object.co_flags
-        self.free_variables = list(self.code_object.co_freevars)
-        self.keyword_only_argument_count = self.code_object.co_kwonlyargcount
-        self.line_table = self.code_object.co_linetable
-        self.lnotab = bytearray(self.code_object.co_lnotab)
-        self.function_name = self.code_object.co_name
-        self.shared_names = list(self.code_object.co_names)
-        # Local variable count is implied by co_varnames
-        self.positional_only_argument_count = self.code_object.co_posonlyargcount
-        self.stack_size = self.code_object.co_stacksize
-        self.shared_variable_names = list(self.code_object.co_varnames)
+        self._load_from_code_object(self.code_object)
 
-        self.__instructions: typing.Optional[typing.List[Instruction]] = None
+    if sys.version_info.major == 3 and sys.version_info.minor == 10:
+
+        def _load_from_code_object(self, obj: types.CodeType):
+            self.argument_count = self.code_object.co_argcount
+            self.cell_variables = list(self.code_object.co_cellvars)
+            self.__raw_code = bytearray(self.code_object.co_code)
+            self.constants = list(self.code_object.co_consts)
+            self.filename = self.code_object.co_filename
+            self.first_line_number = self.code_object.co_firstlineno
+            self.code_flags = self.code_object.co_flags
+            self.free_variables = list(self.code_object.co_freevars)
+            self.keyword_only_argument_count = self.code_object.co_kwonlyargcount
+            self.line_table = self.code_object.co_linetable
+            self.lnotab = bytearray(self.code_object.co_lnotab)
+
+            self.function_name = self.code_object.co_name
+            self.shared_names = list(self.code_object.co_names)
+            # Local variable count is implied by co_varnames
+            self.positional_only_argument_count = self.code_object.co_posonlyargcount
+            self.stack_size = self.code_object.co_stacksize
+            self.shared_variable_names = list(self.code_object.co_varnames)
+
+            self.__instructions: typing.Optional[typing.List[Instruction]] = None
+
+    elif sys.version_info.major == 3 and sys.version_info.minor == 11:
+
+        def _load_from_code_object(self, obj: types.CodeType):
+            self.argument_count = self.code_object.co_argcount
+            self.cell_variables = list(self.code_object.co_cellvars)
+            self.__raw_code = bytearray(self.code_object.co_code)
+            self.constants = list(self.code_object.co_consts)
+            self.filename = self.code_object.co_filename
+            self.first_line_number = self.code_object.co_firstlineno
+            self.code_flags = self.code_object.co_flags
+            self.free_variables = list(self.code_object.co_freevars)
+            self.keyword_only_argument_count = self.code_object.co_kwonlyargcount
+            self.line_table = self.code_object.co_linetable
+            self.lnotab = bytearray(self.code_object.co_lnotab)
+
+            self.function_name = self.code_object.co_name
+            self.shared_names = list(self.code_object.co_names)
+            # Local variable count is implied by co_varnames
+            self.positional_only_argument_count = self.code_object.co_posonlyargcount
+            self.stack_size = self.code_object.co_stacksize
+            self.shared_variable_names = list(self.code_object.co_varnames)
+
+            self.exception_table = bytearray(self.code_object.co_exceptiontable)
+
+            self.__instructions: typing.Optional[typing.List[Instruction]] = None
+
+    else:
+        raise RuntimeError(sys.version_info)
 
     def __repr__(self):
         return f"MutableFunction({self.target})"
@@ -742,32 +960,169 @@ class MutableFunction:
 
         self.__instructions = None
 
-    def create_code_obj(self) -> types.CodeType:
-        if self.__instructions is None:
-            self.get_instructions()
+    if sys.version_info.major == 3 and sys.version_info.minor == 10:
 
-        self.assemble_fast(self.__instructions)
+        def create_code_obj(self) -> types.CodeType:
+            if self.__instructions is None:
+                self.get_instructions()
 
-        return types.CodeType(
-            self.argument_count,
-            self.positional_only_argument_count,
-            self.keyword_only_argument_count,
-            len(self.shared_variable_names),
-            self.stack_size,
-            self.code_flags,
-            bytes(self.raw_code),
-            tuple(self.constants),
-            tuple(self.shared_names),
-            tuple(self.shared_variable_names),
-            self.filename,
-            self.function_name,
-            self.first_line_number,
-            bytes(self.lnotab),
-            tuple(self.free_variables),
-            tuple(self.cell_variables),
-        )
+            self.assemble_fast(self.__instructions)
+
+            return types.CodeType(
+                self.argument_count,
+                self.positional_only_argument_count,
+                self.keyword_only_argument_count,
+                len(self.shared_variable_names),
+                self.stack_size,
+                self.code_flags,
+                bytes(self.raw_code),
+                tuple(self.constants),
+                tuple(self.shared_names),
+                tuple(self.shared_variable_names),
+                self.filename,
+                self.function_name,
+                self.first_line_number,
+                self.get_lnotab(),
+                tuple(self.free_variables),
+                tuple(self.cell_variables),
+            )
+
+    elif sys.version_info.major == 3 and sys.version_info.minor == 11:
+
+        def create_code_obj(self) -> types.CodeType:
+            if self.__instructions is None:
+                self.get_instructions()
+
+            self.assemble_fast(self.__instructions)
+
+            return types.CodeType(
+                self.argument_count,
+                self.positional_only_argument_count,
+                self.keyword_only_argument_count,
+                len(self.shared_variable_names),
+                self.stack_size,
+                self.code_flags,
+                bytes(self.raw_code),
+                tuple(self.constants),
+                tuple(self.shared_names),
+                tuple(self.shared_variable_names),
+                self.filename,
+                self.function_name,
+                self.function_name,
+                self.first_line_number,
+                self.get_lnotab(),
+                bytes(self.exception_table),
+                tuple(self.free_variables),
+                tuple(self.cell_variables),
+            )
+
+    def get_lnotab(self) -> bytes:
+        items = []
+
+        prev_line = self.first_line_number
+        count_since_previous = 0
+
+        for instr in self.__instructions:
+            count_since_previous += 1
+
+            if instr.source_location is None or instr.source_location[0] == prev_line:
+                continue
+
+            offset = instr.source_location[0] - prev_line
+
+            if offset > 127:
+                return bytes()
+
+            if offset < 0:
+                return bytes()
+
+            items.append(count_since_previous)
+            items.append(offset)
+            count_since_previous = 0
+
+        return bytes(items)
+
+    def calculate_max_stack_size(self) -> int:
+        stack_size_table = self.get_max_stack_size_table()
+
+        self.stack_size = max(max(stack_size_table), 0)
+        return self.stack_size
+
+    def get_max_stack_size_table(
+        self, instructions: typing.List[Instruction] = None
+    ) -> typing.List[int]:
+        stack_size_table = [-1] * len(instructions or self.instructions)
+
+        if not instructions:
+            self.prepare_previous_instructions()
+
+        for i, instr in enumerate(instructions or self.instructions):
+            stack = -1
+            if i != 0:
+                prev = (instructions or self.instructions)[i - 1]
+
+                if not prev.has_stop_flow() and not prev.has_unconditional_jump():
+                    stack = stack_size_table[i - 1]
+            else:
+                stack = 0
+
+            if instr.previous_instructions:
+                for instruction in instr.previous_instructions:
+                    if stack_size_table[instruction.offset] != -1:
+                        stack = max(stack, stack_size_table[instruction.offset])
+                        stack += instruction.special_stack_affect_when_followed_by(
+                            instr
+                        )
+
+            if stack != -1:
+                add, sub, _ = instr.get_stack_affect()
+                stack += add
+                stack -= sub
+
+            stack_size_table[i] = stack
+
+        if -1 in stack_size_table:
+            pending = list(filter(lambda e: e[1] == -1, enumerate(stack_size_table)))
+            last_next = -1
+
+            while pending:
+                i, _ = pending.pop(0)
+                instr = (instructions or self.instructions)[i]
+                stack = -1
+
+                if i == last_next:
+                    break
+
+                if not instr.previous_instructions:
+                    print("WARN:", instr)
+                    continue
+
+                for instruction in instr.previous_instructions:
+                    if stack_size_table[instruction.offset] != -1:
+                        stack = max(stack, stack_size_table[instruction.offset])
+                        stack += instruction.special_stack_affect_when_followed_by(
+                            instr
+                        )
+
+                if stack != -1:
+                    add, sub, _ = instr.get_stack_affect()
+                    stack += add
+                    stack -= sub
+
+                    stack_size_table[i] = stack
+
+                    if pending:
+                        last_next = pending[0][0]
+                else:
+                    pending.append((i, -1))
+
+                if last_next == -1:
+                    last_next = i
+
+        return stack_size_table
 
     def reassign_to_function(self):
+        self.calculate_max_stack_size()
         self.target.__code__ = self.create_code_obj()
 
     def decode_instructions(self):
@@ -776,15 +1131,16 @@ class MutableFunction:
         else:
             self.__instructions = []
 
+        line = self.first_line_number
+        lnotab = list(self.lnotab)
+
         extra: int = 0
         for i in range(0, len(self.__raw_code), 2):
             opcode, arg = self.__raw_code[i : i + 2]
 
             if opcode == Opcodes.EXTENDED_ARG:
                 extra = extra * 256 + arg
-                self.__instructions.append(
-                    Instruction(self, i // 2, "NOP", _decode_next=False)
-                )
+                instr = Instruction(self, i // 2, "NOP", _decode_next=False)
 
             else:
                 arg += extra * 256
@@ -793,9 +1149,21 @@ class MutableFunction:
                 if opcode in (Opcodes.FOR_ITER, Opcodes.SETUP_FINALLY):
                     arg += 1
 
-                self.__instructions.append(
-                    Instruction(self, i // 2, opcode, arg=arg, _decode_next=False)
-                )
+                instr = Instruction(self, i // 2, opcode, arg=arg, _decode_next=False)
+
+            if lnotab:
+                lnotab[0] -= 1
+
+                if lnotab[0] == 0:
+                    line_incr = lnotab[1]
+                    del lnotab[:2]
+
+                    line += line_incr
+
+            instr.source_location = (line, None, None)
+
+            self.__instructions.append(instr)
+            # print(instr, instr.source_location)
 
         for i, instruction in enumerate(self.instructions):
             instruction.update_owner(self, i)
@@ -865,6 +1233,7 @@ class MutableFunction:
                     raise
 
         self.assemble_fast(instructions)
+        self.update_instruction_offsets(self.instructions)
         # We do not re-decode, as that would invalidate the instruction instances here
 
     def update_instruction_offsets(self, instructions):
@@ -897,7 +1266,7 @@ class MutableFunction:
                 if instruction.opcode < dis.HAVE_ARGUMENT:
                     arg = 0
                 else:
-                    print(instruction)
+                    print("error", instruction)
 
             if arg >= 256**3:
                 count = 3
@@ -970,12 +1339,23 @@ class MutableFunction:
         while pending_instructions:
             instruction = pending_instructions.pop()
 
+            if not isinstance(instruction, Instruction):
+                print(f"Invalid task instruction: {instruction}")
+                continue
+
+            if not isinstance(instruction, Instruction):
+                raise ValueError(instruction)
+
             # If we visited it, we can skip
             if instruction in visited:
                 continue
 
             # Walk over the instructions as long as we have not met them
             while instruction not in visited:
+                if not isinstance(instruction, Instruction):
+                    print(f"Invalid task instruction: {instruction}")
+                    break
+
                 # If it branches off, it needs to be visited later on
                 if instruction.has_jump():
                     if instruction.arg_value is None:
@@ -999,7 +1379,12 @@ class MutableFunction:
                     instruction.update_owner(self, instruction.offset)
 
                 # The next instruction MUST be set if it does NOT end the control flow
-                assert instruction.next_instruction is not None, instruction
+                if instruction.next_instruction is None:
+                    print("---- start dump")
+                    for instr in instructions:
+                        print(instr)
+                    print("---- end dump")
+                    raise RuntimeError(f"next instruction is None: {instruction}")
 
                 instruction = instruction.next_instruction
 
@@ -1013,6 +1398,7 @@ class MutableFunction:
                 instruction.next_instruction is not None
                 and instruction.offset + 1 != instruction.next_instruction.offset
                 and not instruction.has_unconditional_jump()
+                and not instruction.has_stop_flow()
             ):
                 # todo: do we want to dynamic use assemble_instructions_from_tree()?
                 raise LinearCodeConstraintViolationException(
@@ -1058,12 +1444,14 @@ class MutableFunction:
                         root.next_instruction = instruction
                         missing -= 1
 
-                    # And now, insert the "NOP"'s required after the previous for the things
-                    for delta in range(missing):
-                        nop = Instruction(self, previous.offset + delta + 1, "NOP")
-                        nop.next_instruction = instruction
-                        previous.next_instruction = nop
-                        previous = nop
+                    previous.insert_after(
+                        [
+                            Instruction(self, previous.offset + i + 1, Opcodes.NOP)
+                            for i in range(missing)
+                        ]
+                    )
+
+                nop_count = 0
 
             previous = instruction
 
@@ -1112,6 +1500,9 @@ class MutableFunction:
 
         self.__raw_code.clear()
         for i, instruction in enumerate(self.__instructions):
+            if instruction.opcode > 255:
+                raise ValueError(f"invalid instruction at result level: {instruction}")
+
             arg = instruction.get_arg()
 
             if arg > 255:
@@ -1186,6 +1577,12 @@ class MutableFunction:
         self.shared_variable_names.append(variable_name)
         return len(self.shared_variable_names) - 1
 
+    def allocate_shared_cell(self, name: str):
+        if name in self.cell_variables:
+            return self.cell_variables.index(name)
+        self.cell_variables.append(name)
+        return len(self.cell_variables) - 1
+
     def dump_info(self, file: str):
         data = {
             "instructions": [
