@@ -2,10 +2,15 @@ import dis
 import inspect
 import math
 import os
+import re
+import string
+import struct
 import types
 import typing
 import builtins
+import random
 
+from bytecodemanipulation.MutableFunctionHelpers import Guarantees
 from bytecodemanipulation.MutableFunction import MutableFunction
 from bytecodemanipulation.Opcodes import Opcodes
 from bytecodemanipulation.optimiser_util import inline_const_value_pop_pairs
@@ -45,7 +50,10 @@ class _OptimisationContainer:
             target.run_optimisers(False)
 
     @classmethod
-    def get_for_target(cls, target):
+    def get_for_target(
+        cls,
+        target: types.FunctionType | types.MethodType | typing.Type | types.ModuleType,
+    ):
 
         if hasattr(target, "_OPTIMISER_CONTAINER"):
             return target._OPTIMISER_CONTAINER
@@ -59,12 +67,15 @@ class _OptimisationContainer:
 
         try:
             target._OPTIMISER_CONTAINER = container
-        except (AttributeError,  TypeError):
+        except (AttributeError, TypeError):
             cls._CUSTOM_TARGETS[target] = container
 
         return container
 
-    def __init__(self, target: types.FunctionType | types.MethodType | typing.Type):
+    def __init__(
+        self,
+        target: types.FunctionType | types.MethodType | typing.Type | types.ModuleType,
+    ):
         self.target = target
         self.parents: typing.List["_OptimisationContainer"] = []
         self.children: typing.List["_OptimisationContainer"] = []
@@ -126,22 +137,44 @@ class _OptimisationContainer:
             typing.Type[Exception] | Exception
         ] | None = None
 
-        self.specializations: typing.List[typing.Callable[[SpecializationContainer], None]] = []
+        self.specializations: typing.List[
+            typing.Callable[[SpecializationContainer], None]
+        ] = []
 
         self.is_optimized = False
 
         # Add the children to self
         if isinstance(self.target, type):
             for key, e in self.target.__dict__.items():
-                if isinstance(e, (type(guarantee_builtin_names_are_protected), staticmethod, classmethod, type)):
+                if isinstance(
+                    e,
+                    (
+                        type(guarantee_builtin_names_are_protected),
+                        staticmethod,
+                        classmethod,
+                        type,
+                    ),
+                ):
                     if isinstance(e, (staticmethod, classmethod)):
                         func = e.__func__
                     else:
                         func = e
 
-                    if func.__qualname__.startswith(self.target.__qualname__+".") and func.__module__ == self.target.__module__:
+                    if (
+                        func.__qualname__.startswith(self.target.__qualname__ + ".")
+                        and func.__module__ == self.target.__module__
+                    ):
                         self.children.append(self.get_for_target(e))
 
+        self.guarantees: typing.List[Guarantees.AbstractGuarantee] | None = None
+
+    def add_guarantee(self, guarantee: Guarantees.AbstractGuarantee):
+        if self.guarantees is None:
+            self.guarantees = []
+
+        self.guarantees.append(guarantee)
+        return self
+
     def is_attribute_static(self, name: str):
         return self.is_static or name in self.static_attributes
 
@@ -196,7 +229,7 @@ class _OptimisationContainer:
         if DISABLE_OPTIMISATION_APPLY:
             return
 
-        print("opt", self.target)
+        # print("opt", self.target)
         from bytecodemanipulation.optimiser_util import apply_specializations
 
         if self.children:
@@ -208,7 +241,9 @@ class _OptimisationContainer:
         # resolve the lazy types
         self._resolve_lazy_references()
 
-        if not hasattr(self.target, "__code__") and not isinstance(self.target, (classmethod, staticmethod)):
+        if not hasattr(self.target, "__code__") and not isinstance(
+            self.target, (classmethod, staticmethod)
+        ):
             return
 
         # Create mutable wrapper around the target
@@ -289,7 +324,10 @@ class _OptimisationContainer:
                 if key not in self.dereference_local_var_attr_type:
                     self.dereference_local_var_attr_type[key] = lazy()
 
-            if self.dereference_return_type is None and self.lazy_return_type is not None:
+            if (
+                self.dereference_return_type is None
+                and self.lazy_return_type is not None
+            ):
                 self.dereference_return_type = self.lazy_return_type()
 
             for key, lazy in self.lazy_return_attr_type.items():
@@ -359,6 +397,10 @@ class _OptimisationContainer:
 
 
 _OptimisationContainer.get_for_target(typing).is_static = True
+_OptimisationContainer.get_for_target(random).is_static = True
+_OptimisationContainer.get_for_target(string).is_static = True
+_OptimisationContainer.get_for_target(re).is_static = True
+_OptimisationContainer.get_for_target(struct).is_static = True
 _OptimisationContainer.get_for_target(math).is_static = True
 _OptimisationContainer.get_for_target(os).is_static = True
 
@@ -541,6 +583,7 @@ def guarantee_exact_return_type(
         container.lazy_return_type = (
             lazy_data_type if lazy_data_type is not None else lambda: data_type
         )
+        container.add_guarantee(Guarantees.ReturnType(data_type or lazy_data_type))
 
         return target
 
@@ -640,6 +683,7 @@ def guarantee_constant_result():
     def annotate(target):
         container = _OptimisationContainer.get_for_target(target)
         container.is_constant_op = True
+        container.add_guarantee(Guarantees.RESULT_IS_CONSTANT)
 
         return target
 
