@@ -16,7 +16,7 @@ def specialize_typing_cast(container: SpecializationContainer):
     data_type, value = container.get_arg_specifications()
 
     if ASSERT_TYPE_CASTS:
-        # todo: check type
+        # todo: add check type
         pass
 
     container.replace_call_with_arg(value)
@@ -86,15 +86,141 @@ def create_empty_tuple(container: SpecializationContainer):
         container.replace_with_constant_value(tuple())
 
 
+def _check_int(c) -> bool:
+    if isinstance(c, int):
+        return True
+    if isinstance(c, float):
+        return int(c) == c
+    return False
+
+
 @register(range)
 @register(random.randrange)
 @register(random.Random.randrange)
 def specialize_range_3rd_argument(container: SpecializationContainer):
     args = container.get_arg_specifications()
+    sources = [arg.get_normalized_data_instr() for arg in args]
 
-    if len(args) != 3: return
+    # Argument length checks and type checks
+    if len(sources) in (1, 2, 3):
+        if sources[0].opcode == Opcodes.LOAD_CONST:
+            container.replace_with_raise_exception_if(
+                not _check_int(sources[0].arg_value),
+                lambda: TypeError(
+                    f"'{type(sources[0].arg_value).__name__}' object cannot be interpreted as an integer"
+                ),
+                arg=0,
+            )
 
-    if args[2] and args[2].get_normalized_data_instr().opcode == Opcodes.LOAD_CONST and args[2].get_normalized_data_instr().arg_value == 1:
+        if len(sources) > 1 and sources[1].opcode == Opcodes.LOAD_CONST:
+            container.replace_with_raise_exception_if(
+                not _check_int(sources[1].arg_value),
+                lambda: TypeError(
+                    f"'{type(sources[1].arg_value).__name__}' object cannot be interpreted as an integer"
+                ),
+                arg=1,
+            )
+
+        if (
+            len(sources) == 2
+            and sources[0].opcode == sources[1].opcode == Opcodes.LOAD_CONST
+            and sources[0].arg_value > sources[1].arg_value
+        ):
+            container.replace_with_constant_value(tuple())
+
+        if len(sources) > 2 and sources[2].opcode == Opcodes.LOAD_CONST:
+            if (
+                not container.replace_with_raise_exception_if(
+                    not _check_int(sources[2].arg_value),
+                    lambda: TypeError(
+                        f"'{type(sources[2].arg_value).__name__}' object cannot be interpreted as an integer"
+                    ),
+                    arg=2,
+                )
+                and sources[2].arg_value == 0
+            ):
+                container.replace_with_raise_exception(
+                    lambda: ValueError("range() arg 3 must not be zero"),
+                    arg=2,
+                )
+
+        if (
+            len(sources) == 3
+            and sources[0].opcode
+            == sources[1].opcode
+            == sources[2].opcode
+            == Opcodes.LOAD_CONST
+            and sources[0].arg_value > sources[1].arg_value
+            and sources[2].arg_value > 0
+        ):
+            container.replace_with_constant_value(tuple())
+
+        if (
+            len(sources) == 3
+            and sources[0].opcode
+            == sources[1].opcode
+            == sources[2].opcode
+            == Opcodes.LOAD_CONST
+            and sources[0].arg_value < sources[1].arg_value
+            and sources[2].arg_value < 0
+        ):
+            container.replace_with_constant_value(tuple())
+
+    elif len(sources) == 0:
+        if container.method_call_descriptor.lookup_method_instr.arg_value == range:
+            container.replace_with_raise_exception(
+                TypeError("range expected at least 1 argument, got 0")
+            )
+        elif (
+            container.method_call_descriptor.lookup_method_instr.arg_value
+            == random.randrange
+        ):
+            container.replace_with_raise_exception(
+                TypeError(
+                    "Random.randrange() missing 1 required positional argument: 'start'"
+                )
+            )
+        elif (
+            container.method_call_descriptor.lookup_method_instr.arg_value
+            == random.Random.randrange
+        ):
+            pass
+        else:
+            raise NotImplementedError
+    else:
+        if container.method_call_descriptor.lookup_method_instr.arg_value == range:
+            container.replace_with_raise_exception(
+                TypeError(f"range expected at most 3 arguments, got {len(sources)}")
+            )
+        elif (
+            container.method_call_descriptor.lookup_method_instr.arg_value
+            == random.randrange
+        ):
+            container.replace_with_raise_exception(
+                TypeError(
+                    f"Random.randrange() takes from 2 to 4 positional arguments but {len(sources)} were given"
+                )
+            )
+        elif (
+            container.method_call_descriptor.lookup_method_instr.arg_value
+            == random.Random.randrange
+        ):
+            container.replace_with_raise_exception(
+                TypeError(
+                    f"Random.randrange() takes from 2 to 4 positional arguments but {len(sources)} were given"
+                )
+            )
+        else:
+            raise NotImplementedError
+
+    if len(args) != 3:
+        return
+
+    if (
+        args[2]
+        and sources[2].opcode == Opcodes.LOAD_CONST
+        and sources[2].arg_value == 1
+    ):
         args[2].discard()
 
 
@@ -104,9 +230,14 @@ def specialize_range_3rd_argument(container: SpecializationContainer):
 def specialize_range_start_0(container: SpecializationContainer):
     args = container.get_arg_specifications()
 
-    if len(args) != 2: return
+    if len(args) != 2:
+        return
 
-    if args[0] and args[0].get_normalized_data_instr().opcode == Opcodes.LOAD_CONST and args[0].get_normalized_data_instr().arg_value == 0:
+    if (
+        args[0]
+        and args[0].get_normalized_data_instr().opcode == Opcodes.LOAD_CONST
+        and args[0].get_normalized_data_instr().arg_value == 0
+    ):
         args[0].discard()
 
 
@@ -147,14 +278,20 @@ def specialize_small_range(container: SpecializationContainer):
 def specialize_all(container: SpecializationContainer):
     args = container.get_arg_specifications()
 
-    if len(args) != 1: return
+    if len(args) != 1:
+        return
 
     create_primitive_arg = args[0].get_normalized_data_instr()
 
-    if create_primitive_arg is None: return
+    if create_primitive_arg is None:
+        return
 
     # todo: can we specialize more?
-    if create_primitive_arg.opcode not in (Opcodes.BUILD_LIST, Opcodes.BUILD_TUPLE, Opcodes.BUILD_SET):
+    if create_primitive_arg.opcode not in (
+        Opcodes.BUILD_LIST,
+        Opcodes.BUILD_TUPLE,
+        Opcodes.BUILD_SET,
+    ):
         return
 
     primitive_creation_args = [
@@ -170,7 +307,8 @@ def specialize_all(container: SpecializationContainer):
 
     for normal, real in primitive_creation_args:
         if normal:
-            if normal.opcode != Opcodes.LOAD_CONST: continue
+            if normal.opcode != Opcodes.LOAD_CONST:
+                continue
 
             if not normal.arg_value:
                 if DISCARD_ALL_ANY_WITHOUT_SIDE_EFFECT_CHECK:
@@ -202,14 +340,20 @@ def specialize_all(container: SpecializationContainer):
 def specialize_any(container: SpecializationContainer):
     args = container.get_arg_specifications()
 
-    if len(args) != 1: return
+    if len(args) != 1:
+        return
 
     create_primitive_arg = args[0].get_normalized_data_instr()
 
-    if create_primitive_arg is None: return
+    if create_primitive_arg is None:
+        return
 
     # todo: can we specialize more?
-    if create_primitive_arg.opcode not in (Opcodes.BUILD_LIST, Opcodes.BUILD_TUPLE, Opcodes.BUILD_SET):
+    if create_primitive_arg.opcode not in (
+        Opcodes.BUILD_LIST,
+        Opcodes.BUILD_TUPLE,
+        Opcodes.BUILD_SET,
+    ):
         return
 
     primitive_creation_args = [
@@ -225,7 +369,8 @@ def specialize_any(container: SpecializationContainer):
 
     for normal, real in primitive_creation_args:
         if normal:
-            if normal.opcode != Opcodes.LOAD_CONST: continue
+            if normal.opcode != Opcodes.LOAD_CONST:
+                continue
 
             if normal.arg_value:
                 if DISCARD_ALL_ANY_WITHOUT_SIDE_EFFECT_CHECK:
@@ -251,3 +396,25 @@ def specialize_any(container: SpecializationContainer):
         create_primitive_arg.change_arg(arg_count)
 
         container.no_special = False
+
+
+@register(sum)
+def specialize_any(container: SpecializationContainer):
+    args = container.get_arg_specifications()
+
+    if len(args) == 1:
+        if args[0].get_normalized_data_instr().opcode == Opcodes.LOAD_CONST:
+            container.replace_with_constant_value(
+                sum(args[0].get_normalized_data_instr().arg_value)
+            )
+        elif args[0].get_normalized_data_instr().opcode in (
+            Opcodes.BUILD_LIST,
+            Opcodes.BUILD_TUPLE,
+            Opcodes.BUILD_SET,
+        ):
+            count = args[0].get_normalized_data_instr().arg
+            args[0].get_normalized_data_instr().change_opcode(Opcodes.NOP)
+            container.replace_call_with_opcodes(
+                [Instruction(None, -1, Opcodes.BINARY_ADD) for _ in range(count - 1)],
+                leave_args_on_stack=True,
+            )
