@@ -195,7 +195,7 @@ def inline_constant_method_invokes(mutable: MutableFunction) -> bool:
                         for i in range(instr.arg - 1, -1, -1)
                     ]
 
-                    if all(instr.opcode == Opcodes.LOAD_CONST for instr in args):
+                    if all(ins.opcode == Opcodes.LOAD_CONST for ins in args):
                         result = function(*(e.arg_value for e in args))
 
                         instr.change_opcode(Opcodes.LOAD_CONST)
@@ -345,9 +345,16 @@ def remove_branch_on_constant(mutable: MutableFunction) -> bool:
     dirty = False
 
     for instruction in mutable.instructions:
-        if instruction.has_jump() and not instruction.has_unconditional_jump() and instruction.opcode not in (Opcodes.SETUP_FINALLY, Opcodes.SETUP_WITH):
+        if (
+            instruction.has_jump()
+            and not instruction.has_unconditional_jump()
+            and instruction.opcode not in (Opcodes.SETUP_FINALLY, Opcodes.SETUP_WITH)
+        ):
             if instruction.previous_instructions is None:
-                if instruction.offset == 0 and instruction.opcode == Opcodes.SETUP_FINALLY:
+                if (
+                    instruction.offset == 0
+                    and instruction.opcode == Opcodes.SETUP_FINALLY
+                ):
                     continue
 
                 raise RuntimeError
@@ -415,7 +422,7 @@ def inline_static_attribute_access(mutable: MutableFunction) -> bool:
                     instruction.change_opcode(Opcodes.LOAD_CONST)
                     instruction.change_arg_value(getattr(source, attr_name))
 
-                    print(instruction)
+                    # print(instruction)
 
                     use = next(instruction.trace_stack_position_use(0))
 
@@ -447,12 +454,21 @@ def apply_specializations(mutable: MutableFunction) -> bool:
                 target_func = source.arg_value
 
                 spec = SpecializationContainer()
+                spec.target = mutable
                 spec.underlying_function = target_func
-                spec.method_call_descriptor = MethodCallDescriptor(safe_source, instruction)
-                spec.set_arg_descriptors([
-                    ArgDescriptor(next(instruction.trace_stack_position(arg_id)), instruction.trace_normalized_stack_position(arg_id), arg_id)
-                    for arg_id in range(instruction.arg-1, -1, -1)
-                ])
+                spec.method_call_descriptor = MethodCallDescriptor(
+                    safe_source, instruction
+                )
+                spec.set_arg_descriptors(
+                    [
+                        ArgDescriptor(
+                            next(instruction.trace_stack_position(arg_id)),
+                            instruction.trace_normalized_stack_position(arg_id),
+                            arg_id,
+                        )
+                        for arg_id in range(instruction.arg - 1, -1, -1)
+                    ]
+                )
 
                 for special in container.specializations:
                     special(spec)
